import java.rmi.*;
import java.rmi.server.*;
import java.net.*;
import java.util.*;

/** Klasse zur Verwaltung der Messages.
  * Diese Klasse implementiert das Interface
  * 'MessageInterface', dass die RMI-Methoden
  * zum Speichern (sendMsg) und Abholen (getMsg)
  * enthaelt.
  * @author Bernhard Fiser 0109815
  * @version 0.1 20/10/2002
  */
public class MessageServer extends UnicastRemoteObject implements MessageInterface
{
	/* Hashmap zur Ablage der Nachrichten
         * mit dem Usernamen als Key. */
	private HashMap	userMap;
	private Date	date;
	/* Name unter dem das RMI-Object registriert wird. */
	private final static String
			rmiName = "a0109815";

	/* Constructor um das MessageServer RMI-Object
	 * zu instantieren. */
	public MessageServer() throws RemoteException
	{
		/* Aufruf des Uebergeordneten Constructors
		 * (ist in diesem Fall zwingend notwendig). */
		super();
		/* Hashmap initialisieren. */
		this.userMap = new HashMap();
		this.date = new Date();
	}

	private void println(String s)
	{
		/* Datum auf aktuelle Zeit setzen. */
		this.date.setTime(System.currentTimeMillis());
		System.out.println(this.date + " " + s);
	}

	/** Methode um ein Nachricht an einen oder mehrere User
	  * zu speichern.
	  * @param String user : User, der die Nachricht verschickt.
	  * @param String[] to : StringArray mit Usern, an die
	  * @param               die Nachricht geschickt wird.
	  * @param String msg :  String mit der Nachricht. */
	public synchronized void sendMsg(String user, String[] to, String msg)
	{
		int		i;
		LinkedList	msgList;
		String		msgElement;

		/* Sicherheitsabfrage, Strings null? */
		if ((user == null) || (to == null) || (msg == null))
			return;
		/* Sicherheitsabfrage, Strings leer? */
		if ((user.trim().length() <= 0) || (msg.trim().length() <= 0) || (to.length <= 0))
			return;

		/* Schleife ueber alle User im Array
		 * an die die Nachricht geschickt werden soll. */
		for (i = 0; i < to.length; i++)
		{
			/* Falls noch keine Nachrichten fuer diesen
			 * User vorhanden ... */
			if ((msgList = (LinkedList) this.userMap.get(to[i].trim().toLowerCase())) == null)
			{
				/* ... neue Nachrichtenliste anlegen ... */
				msgList = new LinkedList();
				/* ... und in die userMap aufnehmen. */
				this.userMap.put(to[i].trim().toLowerCase(), msgList);
			}
			/* MessageString zusammensetzen ... */
			msgElement = new String(user + ": " + msg);
			/* ... und zur LinkedList des Users hinzufuegen. */
			msgList.add(msgElement);
			/* Information auf der Console ausgeben. */
			this.println("Nachricht fuer " + to[i].trim() + " gespeichert.");
		}
	}


	/** Methode um Nachricht(en) fuer einen bestimmten
	  * User abholen.
	  * @param String user : User, fuer den die Nachrichten
	  * @param               abgeholt werden sollen.
	  * @return String[] : StringArray mit den Nachrichten oder
	  * @return            null falls keine Nachrichten vorhanden sind. */
        public synchronized String[] getMsg(String user)
	{
		LinkedList	msgList;

		/* Sicherheitsabfrage, String null? */
		if (user == null)
			return(null);
		/* Sicherheitsabfrage, String leer? */
		if (user.trim().length() <= 0)
			return(null);
		/* Nachrichten fuer diesen User vorhanden? */
		if ((msgList = (LinkedList) this.userMap.get(user.trim().toLowerCase())) == null)
		{
			/* Information auf der Console ausgeben. */
			this.println("Keine Nachricht fuer " + user.trim() + " vorhanden.");
			return(null);
		}
		/* Nachrichten aus der userMap entfernen. */
		this.userMap.remove(user.trim().toLowerCase());
		/* Information auf der Console ausgeben. */
		this.println("Nachricht fuer " + user.trim() + " abgeholt.");
		/* Array mit Nachrichten zurckgeben. */
		return((String[]) msgList.toArray(new String[0]));
	}


	/** Kleine private Methode zum Testen. Diese
	  * Methode dient nur zu Debugzwecken. */
	private static void test()
	{
		String[]	argvSend = {"send", "from", "to", "message"};
		String[]	argvGet = {"get", "to"};

		MessageClient.main(argvSend);
		MessageClient.main(argvGet);
	}


	public static void main(String[] argv)
	{
		MessageServer	obj;
		String		webserver = "localhost";

		// use a Security Manager
		if (System.getSecurityManager() == null)
			System.setSecurityManager(new RMISecurityManager());

		try
		{
			/* Hostname des Rechners holen. */
			webserver = InetAddress.getLocalHost().getHostName();
			/* Information auf Console ausgeben. */
			System.out.println("Webserver: " + webserver);

			/* MessageServer RMI-Object instantiieren. */
			obj = new MessageServer();

			/* MessageServer RMI-Object in der RMI-Registry
			 * registrieren. */
			Naming.rebind("rmi://" + webserver + "/" + rmiName, obj);
			/* Information auf der Console ausgeben. */
			System.out.println(rmiName + " bound in registry on " + webserver);
			obj.println("Server ready, starting Console");
			/* Interner Test des MessageServers. */
			//test();
		}
		catch (Exception e)
		{
			/* Fehlermeldung auf Console ausgeben ... */
			System.err.println("***ERROR: " + e.getMessage());
			/* ... und Server beenden. */
			System.exit(1);
		}
	}
}
